// CE7500eep.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include "CE7500eep.h"
#include <commctrl.h>
#include "resource.h"
#include "windev.h"
#include "ntddndis.h"
#include "nuiouser.h"
#include <stdio.h>
#include <string.h>

#define MAJOR_VER 0
#define MINOR_VER 2
#define DATECODE "12/23/2010"

// SCSRs
#define ETH_ID_REV                  (0x00UL)
#define ETH_FPGA_REV                (0x04UL)
#define ETH_INT_STS                 (0x08UL)
#define ETH_RX_CFG                  (0x0CUL)
#define ETH_TX_CFG                  (0x10UL)
#define ETH_HW_CFG                  (0x14UL)
#define ETH_RX_FIFO_INF             (0x18UL)
#define ETH_TX_FIFO_INF             (0x1CUL)
#define ETH_PM_CTRL                 (0x20UL)
#define ETH_LED_GPIO_CFG            (0x24UL)
#define ETH_GPIO_CFG                (0x28UL)
#define ETH_AFC_CFG                 (0x2CUL)
#define ETH_E2P_CMD                 (0x30UL)
#define ETH_E2P_DATA                (0x34UL)
#define ETH_BURST_CAP               (0x38UL)
#define ETH_STRAP_DEBUG             (0x3CUL)
#define ETH_DP_SEL                  (0x40UL)
#define ETH_DP_CMD                  (0x44UL)
#define ETH_DP_ADDR                 (0x48UL)
#define ETH_DP_DATA0                (0x4CUL)
#define ETH_DP_DATA1                (0x50UL)
#define ETH_GPIO_WAKE               (0x64UL)
#define ETH_INT_EP_CTL              (0x68UL)
#define ETH_BULK_IN_DLY             (0x6CUL)

// MAC CSRs
#define MAC_CR                      (0x100UL)
#define ADDRH                       (0x104UL)
#define ADDRL                       (0x108UL)
#define HASHH                       (0x10CUL)
#define HASHL                       (0x110UL)
#define MII_ADDR                    (0x114UL)
#define MII_DATA                    (0x118UL)
#define FLOW                        (0x11CUL)
#define VLAN1                       (0x120UL)
#define VLAN2                       (0x124UL)
#define WUFF                        (0x128UL)
#define WUCSR                       (0x12CUL)
#define ETH_COE_CR                  (0x130UL)

////////////////////////////////////
// PHY Definitions
////////////////////////////////////
#define PHY_BMC_REG                 0
#define PHY_BMS_REG                 1     /* Basic Mode Status Register */
#define PHY_ID1_REG                 2     /* Phy Identifier register 1 */
#define PHY_ID2_REG                 3     /* Phy Identifier register 2 */
#define PHY_ANA_REG                 4
#define PHY_ANLPA_REG               5 
#define PHY_ANE_REG                 6     /* Auto-Negotiation Expansion Reg */
#define PHY_THERA_MODE_CTL_STS      17
#define PHY_THERA_SPECIAL_MODES     18
#define PHY_THERA_CTL_STS_IND       27
#define PHY_THERA_INT_SOURCE        29
#define PHY_THERA_INT_MASK          30
#define PHY_THERA_SPECIAL_REG       31

typedef struct _SCSR_Disp
{
    char name[50];
    DWORD offset;
}SCSR_Disp;

SCSR_Disp LanRegSet[] = {"ETH_ID_REV      ", ETH_ID_REV,
                         "ETH_FPGA_REV    ", ETH_FPGA_REV,
                         "ETH_INT_STS     ", ETH_INT_STS,
                         "ETH_RX_CFG      ", ETH_RX_CFG,
                         "ETH_TX_CFG      ", ETH_TX_CFG,
                         "ETH_HW_CFG      ", ETH_HW_CFG,
                         "ETH_RX_FIFO_INF ", ETH_RX_FIFO_INF,
                         "ETH_TX_FIFO_INF ", ETH_TX_FIFO_INF,
                         "ETH_PM_CTRL     ", ETH_PM_CTRL,
                         "ETH_LED_GPIO_CFG", ETH_LED_GPIO_CFG,
                         "ETH_GPIO_CFG    ", ETH_GPIO_CFG,
                         "ETH_AFC_CFG     ", ETH_AFC_CFG,
                         "ETH_E2P_CMD     ", ETH_E2P_CMD,
                         "ETH_BURST_CAP   ", ETH_BURST_CAP,
                         "ETH_STRAP_DEBUG ", ETH_STRAP_DEBUG,
                         "ETH_DP_SEL      ", ETH_DP_SEL,
                         "ETH_DP_CMD      ", ETH_DP_CMD,
                         "ETH_DP_ADDR     ", ETH_DP_ADDR,
                         "ETH_DP_DATA0    ", ETH_DP_DATA0,
                         "ETH_DP_DATA1    ", ETH_DP_DATA1,
                         "ETH_GPIO_WAKE   ", ETH_GPIO_WAKE,
                         "ETH_INT_EP_CTL  ", ETH_INT_EP_CTL,
                         "ETH_BULK_IN_DLY ", ETH_BULK_IN_DLY};

SCSR_Disp MacRegSet[] = {"MAC_CR    ", MAC_CR,
                         "ADDRH     ", ADDRH,
                         "ADDRL     ", ADDRL,
                         "HASHH     ", HASHH,
                         "HASHL     ", HASHL,
                         "MII_ADDR  ", MII_ADDR,
                         "MII_DATA  ", MII_DATA,
                         "FLOW      ", FLOW,
                         "VLAN1     ", VLAN1,
                         "VLAN2     ", VLAN2,
                         "WUFF      ", WUFF,
                         "WUCSR     ", WUCSR,
                         "ETH_COE_CR", ETH_COE_CR};

SCSR_Disp PhyRegSet[] = {"PHY_BMC_REG            ", PHY_BMC_REG,
                         "PHY_BMS_REG            ", PHY_BMS_REG,
                         "PHY_ID1_REG            ", PHY_ID1_REG,
                         "PHY_ID2_REG            ", PHY_ID2_REG,
                         "PHY_ANA_REG            ", PHY_ANA_REG,
                         "PHY_ANLPA_REG          ", PHY_ANLPA_REG,
                         "PHY_ANE_REG            ", PHY_ANE_REG,
                         "PHY_THERA_MODE_CTL_STS ", PHY_THERA_MODE_CTL_STS,
                         "PHY_THERA_SPECIAL_MODES", PHY_THERA_SPECIAL_MODES,
                         "PHY_THERA_CTL_STS_IND  ", PHY_THERA_CTL_STS_IND,
                         "PHY_THERA_INT_SOURCE   ", PHY_THERA_INT_SOURCE,
                         "PHY_THERA_INT_MASK     ", PHY_THERA_INT_MASK,
                         "PHY_THERA_SPECIAL_REG  ", PHY_THERA_SPECIAL_REG};

BOOL SendE2pCmd(HANDLE	hAdapter, PNDISUIO_QUERY_OID queryOID, DWORD cmd);
BOOL ReadE2p(HANDLE	hAdapter, PNDISUIO_QUERY_OID queryOID, WORD addr, BYTE *val);
BOOL WriteE2p( HANDLE	hAdapter, PNDISUIO_QUERY_OID queryOID, WORD addr, BYTE *val);
BOOL ReadBlockE2p( HANDLE	hAdapter, PNDISUIO_QUERY_OID queryOID, DWORD addr, DWORD length, BYTE *buf);
BOOL WriteBlockE2p( HANDLE	hAdapter, PNDISUIO_QUERY_OID queryOID, DWORD addr, DWORD length, BYTE *buf);
BOOL EraseE2p(HANDLE	hAdapter, PNDISUIO_QUERY_OID queryOID);
BOOL LanReadRegDW(HANDLE	hAdapter,  PNDISUIO_QUERY_OID queryOID, DWORD dwOffset, DWORD *dwVal);
BOOL LanWriteRegDW( HANDLE	hAdapter, PNDISUIO_QUERY_OID queryOID, DWORD dwOffset, DWORD *dwVal);
BOOL PhyReadRegW( HANDLE	hAdapter, PNDISUIO_QUERY_OID queryOID, DWORD dwOffset, DWORD *wVal);
BOOL PhyWriteRegW(HANDLE	hAdapter, PNDISUIO_QUERY_OID queryOID, DWORD dwOffset, DWORD *wVal);
BOOL GetDriverVersion(HANDLE	hAdapter, PNDISUIO_QUERY_OID queryOID, void *Buf);
BOOL GetE2pSize ( HANDLE	hAdapter, PNDISUIO_QUERY_OID queryOID, DWORD*E2pSize);

BOOL SendE2pCmd(HANDLE	hAdapter, PNDISUIO_QUERY_OID queryOID, DWORD cmd)
{
    	BOOL bRc;
	DWORD	dwOutSize;
    	BYTE * p= &(queryOID->Data[0]);
		
    	memset((void *)queryOID->Data,0,(sizeof(queryOID->Data)+sizeof(queryOID->buf)));	

	*(DWORD *)p=cmd;
	
	queryOID->Oid = OID_NDIS_SMSC_SEND_E2P_COMMAND;				
	bRc=DeviceIoControl(hAdapter,
			IOCTL_NDISUIO_SET_OID_VALUE, 
			(LPVOID)queryOID,
			sizeof(NDISUIO_QUERY_OID) + sizeof(DWORD),
			(LPVOID)queryOID,
			sizeof(NDISUIO_QUERY_OID) + sizeof(DWORD),
			&dwOutSize, NULL);
    	if ( !bRc )
    	{
        	printf ( "Error in DeviceIoControl (OID_NDIS_SMSC_SEND_E2P_COMMAND) : %d\n", GetLastError());
       	return FALSE;
    	}
    	return TRUE;
}

BOOL ReadE2p(HANDLE	hAdapter, PNDISUIO_QUERY_OID queryOID, WORD addr, BYTE *val)
{
   	BOOL bRc;
    	DWORD	dwOutSize;
	BYTE * p= &(queryOID->Data[0]);
	
    	memset((void *)queryOID->Data,0,(sizeof(queryOID->Data)+sizeof(queryOID->buf)));

	*(WORD *)p=addr;
				
	queryOID->Oid = OID_NDIS_SMSC_READ_E2P;				
	bRc=DeviceIoControl(hAdapter,
			IOCTL_NDISUIO_QUERY_OID_VALUE, 
			(LPVOID)queryOID,
			sizeof(NDISUIO_QUERY_OID) + sizeof(DWORD),
			(LPVOID)queryOID,
			sizeof(NDISUIO_QUERY_OID) + sizeof(DWORD),
			&dwOutSize, NULL);
    	if ( !bRc )
    	{
        	printf ( "Error in DeviceIoControl (OID_NDIS_SMSC_READ_E2P) : %d\n", GetLastError());
       	return FALSE;
    	}

    *val = queryOID->Data[0];
    return TRUE;
}

BOOL WriteE2p(HANDLE	hAdapter, PNDISUIO_QUERY_OID queryOID, WORD addr, BYTE *val)
{
    	BOOL bRc;
    	BYTE * p= &(queryOID->Data[0]);
       DWORD	dwOutSize;
    	
    	// First Enable Eeprom Write
    	if (SendE2pCmd(hAdapter,queryOID, E2P_CMD_EWEN) == FALSE)
    	{
    		
        	return FALSE;
    	}

	memset((void *)queryOID->Data,0,(sizeof(queryOID->Data)+sizeof(queryOID->buf)));
	
	*(WORD *)p=addr;
	*(BYTE *)((WORD *)p+1)=*val;
	
   					
	queryOID->Oid = OID_NDIS_SMSC_WRITE_E2P;				
	bRc=DeviceIoControl(hAdapter,
			IOCTL_NDISUIO_SET_OID_VALUE, 
			(LPVOID)queryOID,
			sizeof(NDISUIO_QUERY_OID) + sizeof(DWORD),
			(LPVOID)queryOID,
			sizeof(NDISUIO_QUERY_OID) + sizeof(DWORD),
			&dwOutSize, NULL);
    	
    	if ( !bRc )
    	{
        	printf ( "Error in DeviceIoControl (OID_NDIS_SMSC_WRITE_E2P) : %d\n", GetLastError());
        	// Before exit, make sure to reset eeprom write enable
        	SendE2pCmd(hAdapter, queryOID,E2P_CMD_EWDS);
        	return FALSE;
    	}

    	// Before exit, make sure to reset eeprom write enable
    	if (SendE2pCmd(hAdapter, queryOID,E2P_CMD_EWDS) == FALSE)
   	{
   		
        	return FALSE;
    	}

    	return TRUE;
}

BOOL ReadBlockE2p( HANDLE	hAdapter, PNDISUIO_QUERY_OID queryOID, DWORD length, BYTE *buf)
{
    	BOOL bRc;
	BYTE * pE2pSize=&queryOID->Data[0];
    	DWORD	dwOutSize;
		
    	memset((void *)queryOID->Data,0,(sizeof(queryOID->Data)+sizeof(queryOID->buf)));
	*(DWORD *) pE2pSize=length;
					
	queryOID->Oid = OID_NDIS_SMSC_READ_E2P_BLOCK;				
	bRc=DeviceIoControl(hAdapter,
			IOCTL_NDISUIO_QUERY_OID_VALUE, 
			(LPVOID)queryOID,
			sizeof(NDISUIO_QUERY_OID) + sizeof(DWORD),
			(LPVOID)queryOID,
			sizeof(NDISUIO_QUERY_OID) + sizeof(DWORD),
			&dwOutSize, NULL);
    	if ( !bRc )
    	{
        	printf ( "Error in DeviceIoControl (OID_NDIS_SMSC_READ_E2P_BLOCK) : %d\n", GetLastError());
       	return FALSE;
    	}

    	memcpy((void *)buf, (void *)&(queryOID->Data[0]), length);
    	return TRUE;
}

BOOL WriteBlockE2p( HANDLE	hAdapter, PNDISUIO_QUERY_OID queryOID,  DWORD length, BYTE *buf)
{
    	BOOL bRc;
       DWORD	dwOutSize;


	// First Enable Eeprom Write
    	if (SendE2pCmd(hAdapter, queryOID,E2P_CMD_EWEN) == FALSE)
    	{	
    		
   		return FALSE;
    	}
		
 	memset((void *)queryOID->Data,0,(sizeof(queryOID->Data)+sizeof(queryOID->buf)));
	
	memcpy( (void *)&(queryOID->Data[0]), (void *)buf,length);
	
	queryOID->Oid = OID_NDIS_SMSC_WRITE_E2P_BLOCK;				
	bRc=DeviceIoControl(hAdapter,
			IOCTL_NDISUIO_SET_OID_VALUE, 
			(LPVOID)queryOID,
			sizeof(NDISUIO_QUERY_OID) + sizeof(DWORD),
			(LPVOID)queryOID,
			sizeof(NDISUIO_QUERY_OID) + sizeof(DWORD),
			&dwOutSize, NULL);
	
	if ( !bRc )
    	{
        	printf ( "Error in DeviceIoControl (OID_NDIS_SMSC_WRITE_E2P_BLOCK) : %d\n", GetLastError());
        	// Before exit, make sure to reset eeprom write enable
        	SendE2pCmd(hAdapter, queryOID,E2P_CMD_EWDS);
        	return FALSE;
    	}

       // Before exit, make sure to reset eeprom write enable
    	if (SendE2pCmd(hAdapter, queryOID,E2P_CMD_EWDS) == FALSE)
    	{
    		
        	return FALSE;
    	}
    	return TRUE;
}

BOOL EraseE2p(HANDLE	hAdapter, PNDISUIO_QUERY_OID queryOID)
{
   	if ( SendE2pCmd(hAdapter, queryOID,E2P_CMD_EWEN)==FALSE) {
		return FALSE;
	}		
	if ( SendE2pCmd(hAdapter, queryOID,E2P_CMD_ERAL)==FALSE) {
		return FALSE;
	}
	if ( SendE2pCmd(hAdapter, queryOID,E2P_CMD_EWDS)==FALSE) {
		return FALSE;
	}
	return TRUE;
}

BOOL LanReadRegDW(HANDLE	hAdapter, PNDISUIO_QUERY_OID queryOID, DWORD dwOffset, DWORD *dwVal)
{
    	BOOL bRc;
	BYTE * p=&(queryOID->Data[0]);
       DWORD	dwOutSize;
	   
    	memset((void *)queryOID->Data,0,(sizeof(queryOID->Data)+sizeof(queryOID->buf)));	
	*(DWORD *)p=dwOffset;
			
	queryOID->Oid = OID_NDIS_SMSC_LAN_READ_REGS;
	
	bRc=DeviceIoControl(hAdapter,
			IOCTL_NDISUIO_QUERY_OID_VALUE, 
			(LPVOID)queryOID,
			sizeof(NDISUIO_QUERY_OID) + sizeof(DWORD),
			(LPVOID)queryOID,
			sizeof(NDISUIO_QUERY_OID) + sizeof(DWORD),
			&dwOutSize, NULL); 
			
    	if ( !bRc )
   	 {
        	wprintf(L"\nError in DeviceIoControl(),  OID_NDIS_SMSC_LAN_READ_REGS) \n\n\n");
        	return FALSE;
    	}

    	*dwVal = *((DWORD *)&queryOID->Data[0]);
    	return TRUE;
}

BOOL LanWriteRegDW(HANDLE	hAdapter, PNDISUIO_QUERY_OID queryOID, DWORD dwOffset, DWORD *dwVal)
{
    	BOOL bRc;
    	BYTE * p= &(queryOID->Data[0]);
       DWORD	dwOutSize;
	   
    	memset((void *)queryOID->Data,0,(sizeof(queryOID->Data)+sizeof(queryOID->buf)));
	*(DWORD *)p=dwOffset;
	*(DWORD *)((DWORD *)p+1)=*dwVal;
		
	queryOID->Oid = OID_NDIS_SMSC_LAN_WRITE_REGS;
	
	bRc=DeviceIoControl(hAdapter,
			IOCTL_NDISUIO_SET_OID_VALUE, 
			(LPVOID)queryOID,
			sizeof(NDISUIO_QUERY_OID) + sizeof(DWORD),
			(LPVOID)queryOID,
			sizeof(NDISUIO_QUERY_OID) + sizeof(DWORD),
			&dwOutSize, NULL); 
			
    	if ( !bRc )
   	 {
        	wprintf(L"\nError in DeviceIoControl(),  OID_NDIS_SMSC_LAN_WRITE_REGS) \n\n\n");
        	return FALSE;
    	}
    
    	return TRUE;
}

BOOL PhyReadRegW( HANDLE	hAdapter, PNDISUIO_QUERY_OID queryOID, DWORD dwOffset, DWORD *wVal)
{
    	BOOL bRc;
       DWORD	dwOutSize;
    	BYTE * p= &(queryOID->Data[0]);
    
    	memset((void *)queryOID->Data,0,(sizeof(queryOID->Data)+sizeof(queryOID->buf)));
	*(DWORD *)p=dwOffset;
			
	queryOID->Oid = OID_NDIS_SMSC_PHY_READ_REGS;
	
	bRc=DeviceIoControl(hAdapter,
			IOCTL_NDISUIO_QUERY_OID_VALUE, 
			(LPVOID)queryOID,
			sizeof(NDISUIO_QUERY_OID) + sizeof(DWORD),
			(LPVOID)queryOID,
			sizeof(NDISUIO_QUERY_OID) + sizeof(DWORD),
			&dwOutSize, NULL); 
			
    	if ( !bRc )
   	 {
        	wprintf(L"\nError in DeviceIoControl(),  OID_NDIS_SMSC_PHY_READ_REGS) \n\n\n");
        	return FALSE;
    	}

    	*wVal = *(DWORD *)&queryOID->Data[0];
    	return TRUE;
}

BOOL PhyWriteRegW(HANDLE	hAdapter, PNDISUIO_QUERY_OID queryOID, DWORD dwOffset, DWORD *wVal)
{
    	BOOL bRc;
    	BYTE * p= &(queryOID->Data[0]);
       DWORD	dwOutSize;
    	memset((void *)queryOID->Data,0,(sizeof(queryOID->Data)+sizeof(queryOID->buf)));
	*(DWORD *)p=dwOffset;
	*(DWORD *)((DWORD *)p+1)=*wVal;
		
	queryOID->Oid = OID_NDIS_SMSC_LAN_WRITE_REGS;
	
	bRc=DeviceIoControl(hAdapter,
			IOCTL_NDISUIO_SET_OID_VALUE, 
			(LPVOID)queryOID,
			sizeof(NDISUIO_QUERY_OID) + sizeof(DWORD),
			(LPVOID)queryOID,
			sizeof(NDISUIO_QUERY_OID) + sizeof(DWORD),
			&dwOutSize, NULL);
			
    	if ( !bRc )
   	 {
        	wprintf(L"\nError in DeviceIoControl(),  OID_NDIS_SMSC_LAN_WRITE_REGS) \n\n\n");
        	return FALSE;
    	}
    
    	return TRUE;
}


BOOL GetDriverVersion(HANDLE	hAdapter, PNDISUIO_QUERY_OID queryOID,void * pBuf)
{
    	BOOL bRc;
       DWORD	dwOutSize;
		
    	memset((void *)queryOID->Data,0,(sizeof(queryOID->Data)+sizeof(queryOID->buf)));	
		
	queryOID->Oid = OID_NDIS_SMSC_GET_DRIVER_INFO;
	
	bRc=DeviceIoControl(hAdapter,
			IOCTL_NDISUIO_QUERY_OID_VALUE, 
			(LPVOID)queryOID,
			sizeof(NDISUIO_QUERY_OID) + sizeof(DWORD),
			(LPVOID)queryOID,
			sizeof(NDISUIO_QUERY_OID) + sizeof(DWORD),
			&dwOutSize, NULL); 
				
    	if ( !bRc )
   	 {
        	wprintf(L"\nError in DeviceIoControl(),  OID_NDIS_SMSC_GET_DRIVER_INFO) \n\n\n");
        	return FALSE;
    	}

	memcpy(pBuf, (void *)&queryOID->Data[0], 4*sizeof(DWORD));
	
	return TRUE;
}



BOOL GetE2pSize (HANDLE	hAdapter, PNDISUIO_QUERY_OID queryOID, DWORD *pE2pSize)
{
    	      
        // Check to see if it is 256-byte E2P
        BYTE bStart, bEnd, bTemp;

        if (ReadE2p(hAdapter,queryOID, 0, &bStart) == FALSE)
        {
            return FALSE;
        }
	
        if (ReadE2p(hAdapter,queryOID, 256, &bEnd) == FALSE)
        {
            return FALSE;
        }
	
        // Write to 256 the inverse value of bStart
        bTemp = ~bStart;
        if (WriteE2p(hAdapter,queryOID, 256, &bTemp) == FALSE)
        {
            return FALSE;
        }
        
        // Now read locatiion zero again
        if (ReadE2p(hAdapter, queryOID,0, &bTemp) == FALSE)
        {
            return FALSE;
        }

        if (bTemp != bStart) // 256-byte E2P
        {
            // Rewrite correct value into location 0
            if (WriteE2p(hAdapter,queryOID, 0, &bStart) == FALSE)
            {
                return FALSE;
            }
			      
            *pE2pSize = 256;
            return TRUE;
        }
        else // 512-byte E2P
        {
            // Rewrite correct value into location 256
            if (WriteE2p(hAdapter,queryOID, 256, &bEnd) == FALSE)
            {
                return FALSE;
            }

            *pE2pSize = 512;
            return TRUE;
        }
 }


int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
 	// TODO: Place code here.
	
	HANDLE	hAdapter,hFile;
	char	Buffer[1024];
	PTCHAR	pAdapterName;
	DWORD	dwOutSize,ChipId;
	PNDISUIO_QUERY_BINDING	pQueryBinding;
	PNDISUIO_QUERY_OID	queryOID;
	UCHAR	QueryBuffer[sizeof(NDISUIO_QUERY_OID)+sizeof(DWORD)];
	BOOL LoopEndFlag = FALSE;	
	DWORD NofBytes = 0, E2pSize=0; 
	int i=0,key=0;
       DRIVER_VERSION DrvVer;
	 BYTE readBuf[516],writeBuf[516];

	hAdapter = CreateFile(TEXT("UIO1:"), 0x00, 0x00, NULL, OPEN_EXISTING,
				 FILE_ATTRIBUTE_NORMAL | FILE_FLAG_OVERLAPPED,
				 INVALID_HANDLE_VALUE);
	
	if (hAdapter == INVALID_HANDLE_VALUE) {
		wprintf (L"\nCE7500eep: Invalid hAdapter\n\n");
		 return 0;
	}
	pQueryBinding = (PNDISUIO_QUERY_BINDING)&Buffer[0];
		
	memset(pQueryBinding, 0x00, sizeof(NDISUIO_QUERY_BINDING));

	pQueryBinding->BindingIndex = 0;
	
	while(TRUE)	{
	
		
		if (!DeviceIoControl(hAdapter,
				IOCTL_NDISUIO_QUERY_BINDING, pQueryBinding,
				sizeof(NDISUIO_QUERY_BINDING),
				pQueryBinding, 1024, &dwOutSize, NULL))  {
			wprintf (L"\nError at IOCTL_NDISUIO_QUERY_BINDING, GetLastError()= %d\n",GetLastError());
			goto Exit_WinMain;
		}
		
		pAdapterName = (PTCHAR)&Buffer[pQueryBinding->DeviceNameOffset];	
		pAdapterName[(pQueryBinding->DeviceNameLength / sizeof(TCHAR)) - 1] = 0x00;
		if (!wcscmp((const wchar_t *)pAdapterName, (const wchar_t *)L"SMSC75001"))
			break;
		pQueryBinding->BindingIndex++;
	}

	wprintf (L"NDISPWR:: Found adapter [%s]\r\n",pAdapterName);
			
	queryOID = (PNDISUIO_QUERY_OID)&QueryBuffer[0];
	queryOID->ptcDeviceName = pAdapterName;


	if (!GetDriverVersion(hAdapter,queryOID,(void *)&DrvVer.Major)) {
		goto Exit_WinMain;
	}

	if (!LanReadRegDW(hAdapter,queryOID, ETH_ID_REV, &ChipId)) {
		goto Exit_WinMain;
	}

	printf ("Driver Version: V%d.%d.%04d.%04d\n", DrvVer.Major, DrvVer.Minor, DrvVer.Build, DrvVer.Qfe);
	printf ("Chip ID: 0x%x\n", ChipId);
	printf ("Application Version V%d.%d (%s)\n", MAJOR_VER, MINOR_VER, DATECODE);
	printf ("Built at %s on %s)\n\n\n",  __TIME__, __DATE__);
			
	if (!GetE2pSize(hAdapter,queryOID, &E2pSize)) {
		goto Exit_WinMain;
	}
	
			
	while (LoopEndFlag == FALSE)
	{
	     if (key!=10)  {                                   //ignore the line feed  
		wprintf(L"1. Program EEPROM \n");
		wprintf(L"2. Program & Verify EEPROM \n");
	 	wprintf(L"3. read EEPROM\n\n");
		wprintf(L"Q. Exit\n\n\n");
	     	}

		key = getwchar();
		switch (key)
		{
			case '1':
			{
				BYTE *pBuf=writeBuf;
				
				memset(writeBuf, 0x00, sizeof(writeBuf));

				*(DWORD *)pBuf=E2pSize;	
				
				//
				// open the device 
				//
								
				if ((hFile = CreateFile( TEXT("\\Windows\\7500.bin"),
							GENERIC_READ , 
							0,
							NULL,
							OPEN_EXISTING,
							FILE_ATTRIBUTE_NORMAL,
							NULL)) == INVALID_HANDLE_VALUE) {
								wprintf(L"ERROR::Failed to create/open \\Windows\\7500.bin with error %d\n\n",GetLastError());
								goto Exit_WinMain;
				}
					
				if (ReadFile(hFile, 
						 (LPVOID)(writeBuf+4),
						E2pSize,
						&NofBytes,
						0) == FALSE)
				{
						wprintf (L"ERROR::ReadFile Failed\n");
						CloseHandle(hFile);
						goto Exit_WinMain;
				}

				if (NofBytes != E2pSize)
				{
					wprintf (L"ERROR::The file has not been written to the write buffer completely\n");
					wprintf (L"ERROR::files has been written: %d, E2p size: %d\n",NofBytes,E2pSize);
					CloseHandle(hFile);
					goto Exit_WinMain;
				}
				
				CloseHandle(hFile);
					
				if (WriteBlockE2p(hAdapter,queryOID,  E2pSize, writeBuf)== FALSE)
				{		
   					goto Exit_WinMain;
    				}
				
				wprintf(L"Successfully Save \\Windows\\7500.bin to EEPROM, E2p size is %d\n\n\n",E2pSize);
			
			}
				
			break;
			   
			case '2':
				
			{
				DWORD j=0;
				BYTE *pBuf=writeBuf;
				
				memset(writeBuf, 0x00, sizeof(writeBuf));

				*(DWORD *)pBuf=E2pSize;	

				//
				// open the device 
				//
								
				if ((hFile = CreateFile( TEXT("\\Windows\\7500.bin"),
						GENERIC_READ , 
						0,
						NULL,
						OPEN_EXISTING,
						FILE_ATTRIBUTE_NORMAL,
						NULL)) == INVALID_HANDLE_VALUE) {
							wprintf(L"ERROR::Failed to create/open \\Windows\\7500.bin with error %d\n\n",GetLastError());
							goto Exit_WinMain;
				}
					
				if (ReadFile(hFile, 
						(LPVOID)(writeBuf+4),
						E2pSize,
						&NofBytes,
						0) == FALSE)
				{
					wprintf (L"ERROR::ReadFile Failed\n");
					CloseHandle(hFile);
					goto Exit_WinMain;
				}

				if (NofBytes != E2pSize)
				{
					wprintf (L"ERROR::The file has not been written to the write buffer completely\n");
					wprintf (L"ERROR::files has been written: %d, E2p size: %d\n",NofBytes,E2pSize);
					CloseHandle(hFile);
					goto Exit_WinMain;
				}

				CloseHandle(hFile);
				
				wprintf(L" Save \\Windows\\7500.bin to EEPROM..................\n");
				if (WriteBlockE2p(hAdapter,queryOID,  E2pSize, writeBuf)== FALSE)
				{		
   					goto Exit_WinMain;
    				}
			
				wprintf(L" Verify..................\n");
				
				memset(readBuf, 0x00, sizeof(readBuf));

				if (ReadBlockE2p(hAdapter,queryOID, E2pSize, readBuf)== FALSE)
				{		
   					goto Exit_WinMain;
    				}
					   
				for (j=0; j<E2pSize; j++) {

					if  (readBuf[j+4] !=  writeBuf[j+4])
					{
						wprintf(L" Verify failed at %d\n\n\n",j);
						break;
					}

				}

				wprintf(L"Successfully Program & Verify \\Windows\\7500.bin to EEPROM, E2p size is %d\n\n\n",E2pSize);
												
			}
			break;

			case '3':	
				
				{
					memset(readBuf, 0x00, sizeof(readBuf));
					
					if (ReadBlockE2p(hAdapter,queryOID,  E2pSize, readBuf)== FALSE)
					{		
   					goto Exit_WinMain;
					}

						   
					//
					// open the device 
					//
									
					if ((hFile = CreateFile( TEXT("\\Windows\\7500.bin"),
								GENERIC_READ | GENERIC_WRITE,
								0,
								NULL,
								OPEN_ALWAYS,
								FILE_ATTRIBUTE_NORMAL,
								NULL)) == INVALID_HANDLE_VALUE) {
								   	wprintf(L"ERROR::Failed to create/open \\Windows\\7500.bin with error %d\n",GetLastError());
									goto Exit_WinMain;
					}
					

					 if (WriteFile(hFile, 
								  (LPVOID)(readBuf+4),
								  E2pSize,
								  &NofBytes,
								  0) == FALSE)
					{
						wprintf (L"ERROR::WriteFile Failed\n");
						CloseHandle(hFile);
						 goto Exit_WinMain;
					}

					if (NofBytes != E2pSize)
					{
						wprintf (L"ERROR::Write buffer has not been written to the file completely\n\n\n");
						wprintf (L"ERROR::files has been written: %d, E2p size: %d\n",NofBytes,E2pSize);
					} else {
						
						wprintf(L"Successfully Save  EEPROM to file \\Windows\\7500.bin, E2p size is %d\n\n\n",E2pSize);
						
					}
					CloseHandle(hFile);
				}
				break;
				
			case 'q':
            		case 'Q':
                		LoopEndFlag = TRUE;
				wprintf(L"Exit CE7500eep\n\n\n");
				break;
			default:
				if (key != 10)
					wprintf(L"Invalid key, try it again!!!\n\n\n");
				break;
			}
		
		}
Exit_WinMain:

	CloseHandle(hAdapter);	
	return 0;
}


